/**
 * ATKIS-EDBS-Reader This part to convert from temporary to final Oracle database
 * Copyright (C) 2000  Stephan Falke, Carsten Kleiner, University of Hannover
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Ein Paket zum Erstellen und Laden einer Datenbank fr rumliche Daten
 * @author <A href="mailto:fad@gmx.ch">Stephan Falke</A> 
 */
CREATE OR REPLACE PACKAGE fad 
	AUTHID CURRENT_USER AS

	/**
	 * Tabellenname Folientabelle
	 */
	layer_tn VARCHAR2(5) := 'layer';

	/**
	 * Tabellenname Attributtyptabelle
	 */
	attribute_types_tn VARCHAR2(15) := 'attribute_types';

	/**
	 * Tabellenname der Attributwertetabelle
	 */
	attribute_values_tn VARCHAR2(16) := 'attribute_values';

	/**
	 * Tabellenname der Objektartentabelle
	 */
	object_race_tn VARCHAR2(11) := 'object_race';

	/**
	 * Tabellenname der Tabelle der Zieldatenbank der Objekte
	 */
	object_tn VARCHAR2(8) := 'objects';

	/**
	 * Tabellenname der Tabelle der Zieldatenbank der Objektbezeichnungen
	 */
	object_name_tn VARCHAR2(12) := 'object_names';

	/**
	 * Tabellenname der Tabelle der Zieldatenbank der Objektgeometrien
	 */
	object_geometry_tn VARCHAR2(17) := 'object_geometries';

	/**
	 * Tabellenname der Tabelle der Daten der fehlerhaften Objektgeometrien
	 */
	bad_geometries_tn VARCHAR2(17) := 'bad_geometries';

	/**
	 * Tabellenname der Tabelle der Zieldatenbank der berfhrungen
	 */
	above_tn VARCHAR2(5) := 'above';

	/**
	 * Tabellenname der Tabelle der Zieldatenbank der komplexen Objekte
	 */
	complex_object_tn VARCHAR2(15) := 'complex_objects';

	/**
	 * Tabellenname der Tabelle der Zieldatenbank der Attribute
	 */
	attribute_tn VARCHAR2(10) := 'attributes';

	/**
	 * Typname des Typs, der einen Eintrag in die Tabelle der Gemetrieangaben
	 * der Tabelle der Objektbezeichungen beschreibt.
	 */
	name_geo_entry_typ_tn VARCHAR2(18) := 'name_geo_entry_typ';

	/**
	 * Typname des Typs, der die Tabelle der Geometrieangeben der Objektbezeichnungen
	 * beschreibt.
	 */
	name_geo_table_typ_tn VARCHAR2(18) := 'name_geo_table_typ';

	/**
	 * Maximaler Index der Tabellennamen der Zieldatenbank
	 */
	max_table_index NUMBER := 30;
	
	/**
	 * erzeugt die Tabellen einer Zieldatenbank
	 * @my_no der Index in den Tabellennamen
	 */
	PROCEDURE create_tables (my_no VARCHAR2);

	/**
	 * lscht die Tabellen einer Zieldatenbank
	 * @my_no der Index der Tabellennamen der zu lschenden Tabellen
	 * @last die Anzahl der zu lschenden Tabellen
	 */
	PROCEDURE drop_tables (my_no VARCHAR2, last NUMBER);
	
	/**
	 * lscht temporren Tabellen
	 * t_index der Index Tabellennamen der zu lschenden Tabellen
	 */
	PROCEDURE drop_temp_tables (t_index VARCHAR2);

	/**
	 * erzeugt eine Zieldatenbank
	 * @return der Index, mit der die Zieldatenbank erstellt wurde
	 */
	FUNCTION create_all RETURN NUMBER;
	
	/**
	 * ldt die Tabelle der Objekte
	 * @param t_object_name Name der temporren Tabelle der Objekte
	 * @param layer_table_name Name der Folientabelle
	 * @param object_race_table_name Name der Objektartentabelle
	 * @param object_name Name der Tabelle der Zieldatenbank der Objekte
	 */ 
	PROCEDURE load_objects (t_object_name VARCHAR2, layer_table_name VARCHAR2, object_race_table_name VARCHAR2, object_name VARCHAR2);

	/**
	 * ldt die Tabelle der Objektbezeichnungen
	 * @param object_name Name der Tabelle der Zieldatenbank der Objekte
	 * @param t_object_data_name Name der temporren Tabelle der 
	 * besonderen Objektinformationen
	 * @param t_object_geo_name Name der temporren Tabelle der Geometrieangaben
	 * der Objekte
	 * @param name_geo_table_typ_name Name des Typs der Tabelle der 
	 * Geometrieangaben der Bezeichnungen
	 */ 
	PROCEDURE load_object_names (object_name VARCHAR2, t_object_data_name VARCHAR2, t_object_geo_name VARCHAR2, name_geo_table_typ_name VARCHAR2);

	/**
	 * ldt die Tabelle der berfhrungen
	 * @param t_object_data_name Name der temporren Tabelle der 
	 * besonderen Objektinformationen
	 * @param above_name Name der Tabelle der Zieldatenbank der berfhrungen
	 */ 
	PROCEDURE load_above (t_object_data_name VARCHAR2, above_name VARCHAR2);
	
	/**
	 * ldt die Tabelle der Zuordnungen zu komplexen Objekten
	 * @param t_object_data_name Name der temporren Tabelle der 
	 * besonderen Objektinformationen
	 * @param complex_object_name Name der Tabelle der Zieldatenbank der 
	 * Zuordnungen zu komplexe Objekte
	 */ 
	PROCEDURE load_complex_object (t_object_data_name VARCHAR2, complex_object_name VARCHAR2);

	/**
	 * ldt die Tabelle der Gemetrieangaben
	 * @param t_object_name Name der temporren Tabelle der Objekte
	 * @param t_line_name Name der temporren Tabelle der Linien
	 * @param t_line_function name Name der temporren Tabelle der Linienfunktionen
	 * @param t_line_param_name Name der temporren Tabelle der Fachparameter 
	 * der Linien
	 * @param t_line_data_name Name der temporren Tabelle der Lageparameter 
	 * der Linien
	 * @param object_gemotry_name Name der Tabelle der Zieldatenbank der
	 * Objektgeometrien
	 * @param t_geo_name Name der temporen Tabelle fr den Linienverfolgungsalgorithmus
	 */ 
	PROCEDURE load_object_geometry (t_object_name VARCHAR2, t_line_name VARCHAR2, t_line_function_name VARCHAR2, t_line_param_name VARCHAR2, t_line_data_name VARCHAR2, object_gemotry_name VARCHAR2, t_geo_name VARCHAR2);

	/**
	 * ldt die Tabelle der Attribute
	 * @param t_attribute Name der temporren Tabelle der Attribute
	 * @param attribute_name Name der Tabelle der Zieldatenbank der Attribute
	 * @param attribute_type_name Name der Formattabelle der Attributtypen
	 */ 
	PROCEDURE load_attribute (t_attribute_name VARCHAR2, attribute_name VARCHAR2, attribute_type_name VARCHAR2);

	/**
	 * erzeugt und ldt die eine Zieldatenbank
	 * @param t_index der Index der temporren Tabellennamen
	 */
	PROCEDURE create_db (t_index VARCHAR2);
END fad;
/
