
/**
 * ATKIS-EDBS-Reader to read EDBS-File, convert and write it to an Oracle database
 * Copyright (C) 2000  Stephan Falke, Carsten Kleiner, University of Hannover
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package parser;

import java.io.*;
/**
 * Diese Klasse ist verantwortlich f&uuml;r das Starten der <tt>SQL*loader</tt>
 * Aufrufe
 * @author <A href="mailto:fad@gmx.ch">Stephan Falke</A>
 */
class Loader {
	
	/**
	 * erzeugt eine Datei mit <tt>SQL</tt>-Anweisungen, die aus der tempor&auml;ren
	 * Datenbank die Zieldatenbank erstellen und die tempor&auml;ren Tabellen 
	 * l&ouml;scht. 
	 * @param tableIndexNo der Index in den Tabellennamen
	 * @return der Name der erstelten Datei. 
	 */
	private static String writeSqlFile (int tableIndexNo) throws Exception {
		String str = Constants.SQL_FILE[0];
		if (tableIndexNo >= 0) {
			str += tableIndexNo;
		}
		str += Constants.SQL_FILE[1];
		try {
			MyFileOutputStream file = new MyFileOutputStream(Constants.SQL_FILE_NAME);
			file.write(str.getBytes());
			return(file.getFilename());
		}
		catch (Exception e) {}
		for (int i = 0; i < Constants.MAX_CONTROL_FILE_COUNT; i++) {
			try {
				MyFileOutputStream file = new MyFileOutputStream(Constants.SQL_FILE_NAME + i);
				file.write(str.getBytes());
				return(file.getFilename());
			}
			catch (Exception e) {}
		}	
		throw new Exception (Constants.COULDNOT_CREATE_SQL_FILE);
	}
	
	/**
	 * Es wir ein <tt>String</tt> erzeugt, mit dem <tt>sqlplus</tt> gestartet wird und 
	 * die Datei, deren Name in <CODE><A href="../parser/ConstantsBase.html#SQL_FILE_NAME">ConstantsBase
	 * </A></CODE> festgelegt ist, gestartet wird.
	 * @param user Benutzername bei er Datenbank
	 * @param password Pa&szlig;wort bei der Datenbank
	 */
	private static String sqlInstruction (String user, String password, String sqlFileName) {
		String command = Constants.SQL_INSTRUCTION[0] + user + Constants.SQL_INSTRUCTION[1] 
						+ password + Constants.SQL_INSTRUCTION[2] + sqlFileName
						+ Constants.SQL_INSTRUCTION[3];
		return(command);
	}
	
	/**
	 * Es wird ein <tt>String</tt> erzeugt, mit dem der <tt>SQL*loader</tt> aufgerufen
	 * werden kann.
	 * @param user Benutzername bei er Datenbank
	 * @param password Pa&szlig;wort bei der Datenbank
	 * @param files die Namen der Dateien. Anzugeben in dieser Reihenfolge: Datendatei,
	 * Kontrolldatei, Log-Datei, Fehlerdatei
	 * @return Befehl f&uuml;r den <tt>SQL*loader</tt>, der die entsprechenden Daten 
	 * laden l&auml;&szlig;t 
	 */
	private static String loaderInstruction (String user, String password, String files[]) {
		String str = Constants.LOADER_INSTRUCTION_BEGIN + user + Constants.LOADER_USER_PASS + password;
		int max = files.length;
		if (max > Constants.LOADER_FILE_TAGS.length) {
			max = Constants.LOADER_FILE_TAGS.length;
		}
		for (int i = 0; i < max; i++) {
			str += Constants.LOADER_FILE_TAGS[i] + files[i];
		}
		str += Constants.LOADER_INSTRUCTION_END;
		return(str);
	}
	   
	/**
	 * Erstellt eine Kontrolldatei f&uuml;r den <tt>SQL*loader</tt> mit
	 * der eine Tabelle mit &uuml;bergebenen Namen und Spaltennamen gef&uuml;llt 
	 * werden soll.
	 * @param fileName der Name der Datendatei, die eingef&uuml;gt werden soll
	 * @param tableName der Name der Tabelle
	 * @param colums die Spaltennamen der Tabelle
	 * @return der Name der erstellten Datei
	 */
	private static String createControlFile (String fileName, String tableName, String colums[]) throws Exception{
		String str = Constants.CONTROL_FILE_STRINGS[0] + Constants.CONTROL_FILE_STRINGS[1] + 
					fileName + Constants.CONTROL_FILE_STRINGS[1] +
					Constants.CONTROL_FILE_STRINGS[2] + tableName + 
					Constants.CONTROL_FILE_STRINGS[3] + Constants.CONTROL_FILE_STRINGS[1] + 
					Constants.TUPEL_DIVIDER + Constants.CONTROL_FILE_STRINGS[1] +
					Constants.CONTROL_FILE_STRINGS[4];
		if (colums.length > 0) {
			str += colums[0];
		}
		else {
			throw new Exception (Constants.NO_COLUMS + tableName);
		}
		for (int i = 1; i < colums.length; i++) {			 
			str += Constants.TUPEL_DIVIDER + colums[i];
		}
		str += Constants.CONTROL_FILE_STRINGS[5];
		try {
			MyFileOutputStream file = new MyFileOutputStream(tableName + Constants.CONTROL_FILE_NAME_ENDING);
			file.write(str.getBytes());
			return(file.getFilename());
		}
		catch (Exception e) {}
		for (int i = 0; i < Constants.MAX_CONTROL_FILE_COUNT; i++) {
			try {
				MyFileOutputStream file = new MyFileOutputStream(tableName + i + Constants.CONTROL_FILE_NAME_ENDING);
				file.write(str.getBytes());
				return(file.getFilename());
			}
			catch (Exception e) {}
		}	
		throw new Exception (Constants.COULDNOT_CREATE_CONTROLER_FILE + tableName);
	}
	
	/**
	 * erzeugt einen Befehl zum L&ouml;schen der tempor&auml;ren Dateien
	 */
	protected static String rmInstruction () {
		String command = Constants.RM_INSTRUCTION[0] + Constants.LOAD_FILE_ENDING
						+ Constants.RM_INSTRUCTION[1] + Constants.CONTROL_FILE_NAME_ENDING
						+ Constants.RM_INSTRUCTION[2];
		return(command);				
	}
	
	/**
	 * startet die Aufrufe des <tt>SQL*loaders</tt>
	 * @param tableIndexNo der Index in den Tabellennamen
	 * @param loadFileName Name der beim Ladevorgang benutzten Dateien
	 */
	protected static void startLoader (int tableIndexNo, String loadFileName[]) {
		String command = new String();
		try {
			for (int i = 0; i < Constants.TEMP_TABLE_COUNT; i++) {
				String controlFileName = createControlFile(Constants.TABLE_NAMES[i] + Constants.LOAD_FILE_ENDING, tableIndexNo >= 0 ? Constants.TABLE_NAMES[i] + tableIndexNo : Constants.TABLE_NAMES[i], Constants.COLUM[i]);
				String files[] = {	controlFileName,
									loadFileName[i],
									Constants.TABLE_NAMES[i] + Constants.LOG_FILE_ENDING,
									Constants.TABLE_NAMES[i] + Constants.BAD_FILE_ENDING};
				command += loaderInstruction(TempTables.name, TempTables.pass, files) + "; ";
			}
			try {
				String sqlFileName = writeSqlFile(tableIndexNo);
				command += sqlInstruction(TempTables.name, TempTables.pass, sqlFileName);
			}
			catch (Exception e) {
				System.out.println(e.getMessage());
			}
			command += rmInstruction();	
			Shell shell = new Shell(command); 
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
		}
		return;
	}
}
